try:
    from mkl_random import *
except ImportError as e:
    raise ImportError("mkl_random not found. Install it with 'conda instal -c intel mkl_random', or get it from 'http://github.com/IntelPython/mkl_random'")
